package schemas

import (
	"github.com/hashicorp/terraform-plugin-framework/datasource/schema"
	"github.com/hashicorp/terraform-plugin-framework/types"
)

func MakeSchemaDataSourceAccount() map[string]schema.Attribute {
	return map[string]schema.Attribute{
		// required attributes
		"account_id": schema.Int64Attribute{
			Required:    true,
			Description: "account id",
		},

		// computed attributes
		"id": schema.StringAttribute{
			Computed: true,
		},
		"dc_location": schema.StringAttribute{
			Computed: true,
		},
		"ckey": schema.StringAttribute{
			Computed: true,
		},
		"acl": schema.ListNestedAttribute{
			Computed: true,
			NestedObject: schema.NestedAttributeObject{
				Attributes: map[string]schema.Attribute{
					"can_be_deleted": schema.BoolAttribute{
						Computed: true,
					},
					"explicit": schema.BoolAttribute{
						Computed: true,
					},
					"guid": schema.StringAttribute{
						Computed: true,
					},
					"right": schema.StringAttribute{
						Computed: true,
					},
					"status": schema.StringAttribute{
						Computed: true,
					},
					"type": schema.StringAttribute{
						Computed: true,
					},
					"user_group_id": schema.StringAttribute{
						Computed: true,
					},
				},
			},
		},
		"company": schema.StringAttribute{
			Computed: true,
		},
		"companyurl": schema.StringAttribute{
			Computed: true,
		},
		"compute_features": schema.ListAttribute{
			Computed:    true,
			ElementType: types.StringType,
		},
		"created_by": schema.StringAttribute{
			Computed: true,
		},
		"created_time": schema.Int64Attribute{
			Computed: true,
		},
		"deactivation_time": schema.Float64Attribute{
			Computed: true,
		},
		"deleted_by": schema.StringAttribute{
			Computed: true,
		},
		"deleted_time": schema.Int64Attribute{
			Computed: true,
		},
		"displayname": schema.StringAttribute{
			Computed: true,
		},
		"guid": schema.Int64Attribute{
			Computed: true,
		},
		"account_name": schema.StringAttribute{
			Computed: true,
		},
		"resource_limits": schema.SingleNestedAttribute{
			Computed: true,
			Attributes: map[string]schema.Attribute{
				"cu_c": schema.Float64Attribute{
					Computed: true,
				},
				"cu_d": schema.Float64Attribute{
					Computed: true,
				},
				"cu_i": schema.Float64Attribute{
					Computed: true,
				},
				"cu_m": schema.Float64Attribute{
					Computed: true,
				},
				"cu_dm": schema.Float64Attribute{
					Computed: true,
				},
				"cu_np": schema.Float64Attribute{
					Computed: true,
				},
				"gpu_units": schema.Float64Attribute{
					Computed: true,
				},
			},
		},
		"send_access_emails": schema.BoolAttribute{
			Computed: true,
		},
		"status": schema.StringAttribute{
			Computed: true,
		},
		"updated_time": schema.Int64Attribute{
			Computed: true,
		},
		"version": schema.Int64Attribute{
			Computed: true,
		},
		"vins": schema.ListAttribute{
			Computed:    true,
			ElementType: types.Int64Type,
		},
		"computes": schema.SingleNestedAttribute{
			Computed: true,
			Attributes: map[string]schema.Attribute{
				"started": schema.Int64Attribute{
					Computed: true,
				},
				"stopped": schema.Int64Attribute{
					Computed: true,
				},
			},
		},
		"machines": schema.SingleNestedAttribute{
			Computed: true,
			Attributes: map[string]schema.Attribute{
				"halted": schema.Int64Attribute{
					Computed: true,
				},
				"running": schema.Int64Attribute{
					Computed: true,
				},
			},
		},
		"vinses": schema.Int64Attribute{
			Computed: true,
		},
		"cpu_allocation_parameter": schema.StringAttribute{
			Computed: true,
		},
		"cpu_allocation_ratio": schema.Float64Attribute{
			Computed: true,
		},
	}
}
