package extnet

import (
	"context"

	"github.com/hashicorp/terraform-plugin-framework-timeouts/datasource/timeouts"
	"github.com/hashicorp/terraform-plugin-framework/datasource"
	"github.com/hashicorp/terraform-plugin-framework/datasource/schema"
	"github.com/hashicorp/terraform-plugin-log/tflog"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/client"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/constants"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/service/cloudapi/extnet/flattens"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/service/cloudapi/extnet/models"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/service/cloudapi/extnet/schemas"
)

// Ensure the implementation satisfies the expected interfaces.
var (
	_ datasource.DataSource = &dataSourceExtNetReservedIP{}
)

func NewDataSourceExtNetReservedIP() datasource.DataSource {
	return &dataSourceExtNetReservedIP{}
}

// dataSourceExtNet is the data source implementation.
type dataSourceExtNetReservedIP struct {
	client *client.Client
}

func (d *dataSourceExtNetReservedIP) Read(ctx context.Context, req datasource.ReadRequest, resp *datasource.ReadResponse) {
	// Read Terraform configuration data into the model
	var state models.DataSourceExtNetReservedIPModel
	resp.Diagnostics.Append(req.Config.Get(ctx, &state)...)
	if resp.Diagnostics.HasError() {
		tflog.Error(ctx, "Read dataSourceExtNetReservedIP: Error get state")
		return
	}
	tflog.Info(ctx, "Read dataSourceExtNetReservedIP: got state successfully")

	// Set timeouts
	readTimeout, diags := state.Timeouts.Read(ctx, constants.Timeout30s)
	resp.Diagnostics.Append(diags...)
	if resp.Diagnostics.HasError() {
		tflog.Error(ctx, "Read dataSourceExtNetReservedIP: Error set timeout")
		return
	}
	tflog.Info(ctx, "Read dataSourceExtNetReservedIP: set timeouts successfully", map[string]any{"readTimeout": readTimeout})

	ctx, cancel := context.WithTimeout(ctx, readTimeout)
	defer cancel()

	// Map response body to schema
	resp.Diagnostics.Append(flattens.ExtNetReservedIPDataSource(ctx, &state, d.client)...)
	if resp.Diagnostics.HasError() {
		tflog.Error(ctx, "Read dataSourceExtNetReservedIP: Error flatten data source extnet_reserved_ip")
		return
	}

	// Set refreshed state
	resp.Diagnostics.Append(resp.State.Set(ctx, &state)...)
	if resp.Diagnostics.HasError() {
		tflog.Error(ctx, "Read dataSourceExtNetReservedIP: Error set state")
		return
	}
	tflog.Info(ctx, "End read dataSourceExtNetReservedIP")
}

func (d *dataSourceExtNetReservedIP) Schema(ctx context.Context, _ datasource.SchemaRequest, resp *datasource.SchemaResponse) {
	resp.Schema = schema.Schema{
		Attributes: schemas.MakeSchemaDataSourceExtNetReservedIP(),
		Blocks: map[string]schema.Block{
			"timeouts": timeouts.Block(ctx),
		},
	}
}

func (d *dataSourceExtNetReservedIP) Metadata(_ context.Context, req datasource.MetadataRequest, resp *datasource.MetadataResponse) {
	resp.TypeName = req.ProviderTypeName + "_extnet_reserved_ip_list"
}

// Configure adds the provider configured client to the data source.
func (d *dataSourceExtNetReservedIP) Configure(ctx context.Context, req datasource.ConfigureRequest, resp *datasource.ConfigureResponse) {
	tflog.Info(ctx, "Get Configure dataSourceExtNetReservedIP")
	d.client = client.DataSource(ctx, &req, resp)
	tflog.Info(ctx, "Getting Configure dataSourceExtNetReservedIP successfully")
}
