package vfpool

import (
	"context"

	"github.com/hashicorp/terraform-plugin-framework-timeouts/datasource/timeouts"
	"github.com/hashicorp/terraform-plugin-framework/datasource"
	"github.com/hashicorp/terraform-plugin-framework/datasource/schema"
	"github.com/hashicorp/terraform-plugin-log/tflog"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/client"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/constants"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/service/cloudapi/vfpool/flattens"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/service/cloudapi/vfpool/models"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/service/cloudapi/vfpool/schemas"
)

// Ensure the implementation satisfies the expected interfaces.
var (
	_ datasource.DataSource = &dataSourceVFPoolList{}
)

func NewDataSourceVFPoolList() datasource.DataSource {
	return &dataSourceVFPoolList{}
}

// dataSourceVFPoolList is the data source implementation.
type dataSourceVFPoolList struct {
	client *client.Client
}

func (d *dataSourceVFPoolList) Read(ctx context.Context, req datasource.ReadRequest, resp *datasource.ReadResponse) {
	// Read Terraform configuration data into the model
	var state models.ListVFPoolModel
	resp.Diagnostics.Append(req.Config.Get(ctx, &state)...)
	if resp.Diagnostics.HasError() {
		tflog.Error(ctx, "Read dataSourceVFPoolList: Error get state")
		return
	}
	tflog.Info(ctx, "Read dataSourceVFPoolList: got state successfully")

	// Set timeouts
	readTimeout, diags := state.Timeouts.Read(ctx, constants.Timeout30s)
	resp.Diagnostics.Append(diags...)
	if resp.Diagnostics.HasError() {
		tflog.Error(ctx, "Read dataSourceVFPoolList: Error set timeout")
		return
	}
	tflog.Info(ctx, "Read dataSourceVFPoolList: set timeouts successfully", map[string]any{
		"readTimeout": readTimeout})

	ctx, cancel := context.WithTimeout(ctx, readTimeout)
	defer cancel()

	// Map response body to schema
	resp.Diagnostics.Append(flattens.VFPoolListDataSource(ctx, &state, d.client)...)
	if resp.Diagnostics.HasError() {
		tflog.Error(ctx, "Read dataSourceVFPoolList: Error flatten data source")
		return
	}

	// Set refreshed state
	resp.Diagnostics.Append(resp.State.Set(ctx, &state)...)
	if resp.Diagnostics.HasError() {
		tflog.Error(ctx, "Read dataSourceVFPoolList: Error set state")
		return
	}
	tflog.Info(ctx, "End read vfpool list")
}

func (d *dataSourceVFPoolList) Schema(ctx context.Context, _ datasource.SchemaRequest, resp *datasource.SchemaResponse) {
	resp.Schema = schema.Schema{
		Attributes: schemas.MakeSchemaDataSourceVFPoolList(),
		Blocks: map[string]schema.Block{
			"timeouts": timeouts.Block(ctx),
		},
	}
}

func (d *dataSourceVFPoolList) Metadata(_ context.Context, req datasource.MetadataRequest, resp *datasource.MetadataResponse) {
	resp.TypeName = req.ProviderTypeName + "_vfpool_list"
}

// Configure adds the provider configured client to the data source.
func (d *dataSourceVFPoolList) Configure(ctx context.Context, req datasource.ConfigureRequest, resp *datasource.ConfigureResponse) {
	tflog.Info(ctx, "Get Configure dataSourceVFPoolList")
	d.client = client.DataSource(ctx, &req, resp)
	tflog.Info(ctx, "Getting Configure dataSourceVFPoolList successfully")
}
