package schemas

import (
	"github.com/hashicorp/terraform-plugin-framework/datasource/schema"
	"github.com/hashicorp/terraform-plugin-framework/types"
)

func MakeSchemaDataSourceAccountListDeleted() map[string]schema.Attribute {
	return map[string]schema.Attribute{
		// optional attributes
		"by_id": schema.Int64Attribute{
			Optional:    true,
			Description: "filter by id",
		},
		"name": schema.StringAttribute{
			Optional:    true,
			Description: "filter by name",
		},
		"acl": schema.StringAttribute{
			Optional:    true,
			Description: "filter by acl",
		},
		"page": schema.Int64Attribute{
			Optional:    true,
			Description: "page number",
		},
		"size": schema.Int64Attribute{
			Optional:    true,
			Description: "page size",
		},
		"sort_by": schema.StringAttribute{
			Optional:    true,
			Description: "sort by one of supported fields, format +|-(field)",
		},

		// computed attributes
		"id": schema.StringAttribute{
			Computed: true,
		},
		"items": schema.ListNestedAttribute{
			Computed: true,
			NestedObject: schema.NestedAttributeObject{
				Attributes: map[string]schema.Attribute{
					"dc_location": schema.StringAttribute{
						Computed: true,
					},
					"ckey": schema.StringAttribute{
						Computed: true,
					},
					"meta": schema.ListAttribute{
						Computed:    true,
						ElementType: types.StringType,
					},
					"acl": schema.ListNestedAttribute{
						Computed: true,
						NestedObject: schema.NestedAttributeObject{
							Attributes: map[string]schema.Attribute{
								"explicit": schema.BoolAttribute{
									Computed: true,
								},
								"guid": schema.StringAttribute{
									Computed: true,
								},
								"right": schema.StringAttribute{
									Computed: true,
								},
								"status": schema.StringAttribute{
									Computed: true,
								},
								"type": schema.StringAttribute{
									Computed: true,
								},
								"user_group_id": schema.StringAttribute{
									Computed: true,
								},
							},
						},
					},
					"compute_features": schema.ListAttribute{
						Computed:    true,
						ElementType: types.StringType,
					},
					"company": schema.StringAttribute{
						Computed: true,
					},
					"companyurl": schema.StringAttribute{
						Computed: true,
					},
					"cpu_allocation_parameter": schema.StringAttribute{
						Computed: true,
					},
					"cpu_allocation_ratio": schema.Float64Attribute{
						Computed: true,
					},
					"created_time": schema.Int64Attribute{
						Computed: true,
					},
					"created_by": schema.StringAttribute{
						Computed: true,
					},
					"deleted_time": schema.Int64Attribute{
						Computed: true,
					},
					"deleted_by": schema.StringAttribute{
						Computed: true,
					},
					"account_id": schema.Int64Attribute{
						Computed: true,
					},
					"guid": schema.Int64Attribute{
						Computed: true,
					},
					"account_name": schema.StringAttribute{
						Computed: true,
					},
					"updated_time": schema.Int64Attribute{
						Computed: true,
					},
					"resource_limits": schema.SingleNestedAttribute{
						Optional: true,
						Computed: true,
						Attributes: map[string]schema.Attribute{
							"cu_c": schema.Float64Attribute{
								Optional: true,
								Computed: true,
							},
							"cu_d": schema.Float64Attribute{
								Optional: true,
								Computed: true,
							},
							"cu_dm": schema.Float64Attribute{
								Optional: true,
								Computed: true,
							},
							"cu_i": schema.Float64Attribute{
								Optional: true,
								Computed: true,
							},
							"cu_m": schema.Float64Attribute{
								Optional: true,
								Computed: true,
							},
							"cu_np": schema.Float64Attribute{
								Optional: true,
								Computed: true,
							},
							"gpu_units": schema.Float64Attribute{
								Optional: true,
								Computed: true,
							},
						},
					},
					"resource_types": schema.ListAttribute{
						Computed:    true,
						ElementType: types.StringType,
					},
					"send_access_emails": schema.BoolAttribute{
						Computed: true,
					},
					"status": schema.StringAttribute{
						Computed: true,
					},
					"uniq_pools": schema.ListAttribute{
						Computed:    true,
						ElementType: types.StringType,
					},
					"version": schema.Int64Attribute{
						Computed: true,
					},
					"vins": schema.ListAttribute{
						Computed:    true,
						ElementType: types.Int64Type,
					},
				},
			},
		},
		"entry_count": schema.Int64Attribute{
			Computed: true,
		},
	}
}
