package models

import (
	"github.com/hashicorp/terraform-plugin-framework-timeouts/datasource/timeouts"
	"github.com/hashicorp/terraform-plugin-framework/attr"
	"github.com/hashicorp/terraform-plugin-framework/types"
)

type RecordDiskModel struct {
	// request fields - required
	DiskId   types.Int64    `tfsdk:"disk_id"`
	Timeouts timeouts.Value `tfsdk:"timeouts"`

	// response fields
	ACL                 types.String           `tfsdk:"acl"`
	BootPartition       types.Int64            `tfsdk:"boot_partition"`
	AccountID           types.Int64            `tfsdk:"account_id"`
	AccountName         types.String           `tfsdk:"account_name"`
	Computes            types.List             `tfsdk:"computes"`
	CreatedTime         types.Int64            `tfsdk:"created_time"`
	DeletedTime         types.Int64            `tfsdk:"deleted_time"`
	DeviceName          types.String           `tfsdk:"devicename"`
	DiskPath            types.String           `tfsdk:"disk_path"`
	Description         types.String           `tfsdk:"desc"`
	DestructionTime     types.Int64            `tfsdk:"destruction_time"`
	GID                 types.Int64            `tfsdk:"gid"`
	GUID                types.Int64            `tfsdk:"guid"`
	ImageID             types.Int64            `tfsdk:"image_id"`
	Images              types.List             `tfsdk:"images"`
	IOTune              *DiskReplicationIOTune `tfsdk:"iotune"`
	IQN                 types.String           `tfsdk:"iqn"`
	Login               types.String           `tfsdk:"login"`
	Milestones          types.Int64            `tfsdk:"milestones"`
	Name                types.String           `tfsdk:"disk_name"`
	Order               types.Int64            `tfsdk:"order"`
	Params              types.String           `tfsdk:"params"`
	ParentID            types.Int64            `tfsdk:"parent_id"`
	Passwd              types.String           `tfsdk:"passwd"`
	PCISlot             types.Int64            `tfsdk:"pci_slot"`
	Pool                types.String           `tfsdk:"pool"`
	PresentTo           types.List             `tfsdk:"present_to"`
	PurgeAttempts       types.Int64            `tfsdk:"purge_attempts"`
	PurgeTime           types.Int64            `tfsdk:"purge_time"`
	Replication         *ItemReplicationModel  `tfsdk:"replication"`
	RealityDeviceNumber types.Int64            `tfsdk:"reality_device_number"`
	ReferenceID         types.String           `tfsdk:"reference_id"`
	ResID               types.String           `tfsdk:"res_id"`
	ResName             types.String           `tfsdk:"res_name"`
	Role                types.String           `tfsdk:"role"`
	SepType             types.String           `tfsdk:"sep_type"`
	SepID               types.Int64            `tfsdk:"sep_id"`
	Shareable           types.Bool             `tfsdk:"shareable"`
	SizeMax             types.Int64            `tfsdk:"size_max"`
	SizeUsed            types.Float64          `tfsdk:"size_used"`
	Snapshots           types.List             `tfsdk:"snapshots"`
	Status              types.String           `tfsdk:"status"`
	StatusReplication   types.String           `tfsdk:"status_replication"`
	TechStatus          types.String           `tfsdk:"tech_status"`
	Type                types.String           `tfsdk:"type"`
	VMID                types.Int64            `tfsdk:"vmid"`
}

type DiskReplicationIOTune struct {
	ReadBytesSec     types.Int64 `tfsdk:"read_bytes_sec"`
	ReadBytesSecMax  types.Int64 `tfsdk:"read_bytes_sec_max"`
	ReadIOPSSec      types.Int64 `tfsdk:"read_iops_sec"`
	ReadIOPSSecMax   types.Int64 `tfsdk:"read_iops_sec_max"`
	SizeIOPSSec      types.Int64 `tfsdk:"size_iops_sec"`
	TotalBytesSec    types.Int64 `tfsdk:"total_bytes_sec"`
	TotalBytesSecMax types.Int64 `tfsdk:"total_bytes_sec_max"`
	TotalIOPSSec     types.Int64 `tfsdk:"total_iops_sec"`
	TotalIOPSSecMax  types.Int64 `tfsdk:"total_iops_sec_max"`
	WriteBytesSec    types.Int64 `tfsdk:"write_bytes_sec"`
	WriteBytesSecMax types.Int64 `tfsdk:"write_bytes_sec_max"`
	WriteIOPSSec     types.Int64 `tfsdk:"write_iops_sec"`
	WriteIOPSSecMax  types.Int64 `tfsdk:"write_iops_sec_max"`
}

type ReplicationModel struct {
	DiskID       types.Int64  `tfsdk:"disk_id"`
	PoolID       types.String `tfsdk:"pool_id"`
	Role         types.String `tfsdk:"role"`
	SelfVolumeID types.String `tfsdk:"self_volume_id"`
	StorageID    types.String `tfsdk:"storage_id"`
	VolumeID     types.String `tfsdk:"volume_id"`
}

type DiskReplicationItemSnapshot struct {
	GUID        types.String `tfsdk:"guid"`
	Label       types.String `tfsdk:"label"`
	ReferenceID types.String `tfsdk:"reference_id"`
	ResID       types.String `tfsdk:"res_id"`
	SnapSetGUID types.String `tfsdk:"snap_set_guid"`
	SnapSetTime types.Int64  `tfsdk:"snap_set_time"`
	TimeStamp   types.Int64  `tfsdk:"timestamp"`
}

var DiskReplicationSnapshot = map[string]attr.Type{
	"guid":          types.StringType,
	"label":         types.StringType,
	"res_id":        types.StringType,
	"snap_set_guid": types.StringType,
	"snap_set_time": types.Int64Type,
	"timeStamp":     types.Int64Type,
}

type ItemComputeModel struct {
	ComputeId   types.String `tfsdk:"compute_id"`
	ComputeName types.String `tfsdk:"compute_name"`
}

var ItemCompute = map[string]attr.Type{
	"compute_id":   types.StringType,
	"compute_name": types.StringType,
}
