package schemas

import (
	"github.com/hashicorp/terraform-plugin-framework/datasource/schema"
	"github.com/hashicorp/terraform-plugin-framework/types"
)

func MakeSchemaDataSourceRG() map[string]schema.Attribute {
	return map[string]schema.Attribute{
		// required attributes
		"rg_id": schema.Int64Attribute{
			Required:    true,
			Description: "resource group id",
		},

		//computed attributes
		"account_id": schema.Int64Attribute{
			Computed: true,
		},
		"account_name": schema.StringAttribute{
			Computed: true,
		},
		"acl": schema.ListNestedAttribute{
			Computed: true,
			NestedObject: schema.NestedAttributeObject{
				Attributes: map[string]schema.Attribute{
					"explicit": schema.BoolAttribute{
						Computed: true,
					},
					"guid": schema.StringAttribute{
						Computed: true,
					},
					"right": schema.StringAttribute{
						Computed: true,
					},
					"status": schema.StringAttribute{
						Computed: true,
					},
					"type": schema.StringAttribute{
						Computed: true,
					},
					"user_group_id": schema.StringAttribute{
						Computed: true,
					},
				},
			},
		},
		"compute_features": schema.ListAttribute{
			Computed:    true,
			ElementType: types.StringType,
		},
		"cpu_allocation_parameter": schema.StringAttribute{
			Computed: true,
		},
		"cpu_allocation_ratio": schema.Float64Attribute{
			Computed: true,
		},
		"created_by": schema.StringAttribute{
			Computed: true,
		},
		"created_time": schema.Int64Attribute{
			Computed: true,
		},
		"def_net_id": schema.Int64Attribute{
			Computed: true,
		},
		"def_net_type": schema.StringAttribute{
			Computed: true,
		},
		"deleted_by": schema.StringAttribute{
			Computed: true,
		},
		"deleted_time": schema.Int64Attribute{
			Computed: true,
		},
		"desc": schema.StringAttribute{
			Computed: true,
		},
		"dirty": schema.BoolAttribute{
			Computed: true,
		},
		"gid": schema.Int64Attribute{
			Computed: true,
		},
		"guid": schema.Int64Attribute{
			Computed: true,
		},
		"id": schema.StringAttribute{
			Computed: true,
		},
		"lock_status": schema.StringAttribute{
			Computed: true,
		},
		"milestones": schema.Int64Attribute{
			Computed: true,
		},
		"name": schema.StringAttribute{
			Computed: true,
		},
		"register_computes": schema.BoolAttribute{
			Computed: true,
		},
		"resource_limits": schema.SingleNestedAttribute{
			Computed: true,
			Attributes: map[string]schema.Attribute{
				"cu_c": schema.Float64Attribute{
					Computed: true,
				},
				"cu_d": schema.Float64Attribute{
					Computed: true,
				},
				"cu_dm": schema.Float64Attribute{
					Computed: true,
				},
				"cu_i": schema.Float64Attribute{
					Computed: true,
				},
				"cu_m": schema.Float64Attribute{
					Computed: true,
				},
				"cu_np": schema.Float64Attribute{
					Computed: true,
				},
				"gpu_units": schema.Float64Attribute{
					Computed: true,
				},
			},
		},
		"resource_types": schema.ListAttribute{
			Computed:    true,
			ElementType: types.StringType,
		},
		"secret": schema.StringAttribute{
			Computed: true,
		},
		"status": schema.StringAttribute{
			Computed: true,
		},
		"updated_by": schema.StringAttribute{
			Computed: true,
		},
		"updated_time": schema.Int64Attribute{
			Computed: true,
		},
		"vins": schema.ListAttribute{
			Computed:    true,
			ElementType: types.Int64Type,
		},
		"computes": schema.ListAttribute{
			Computed:    true,
			ElementType: types.Int64Type,
		},
		"uniq_pools": schema.ListAttribute{
			Computed:    true,
			ElementType: types.StringType,
		},
	}
}
