package vins

import (
	"context"

	"github.com/hashicorp/terraform-plugin-framework-timeouts/datasource/timeouts"
	"github.com/hashicorp/terraform-plugin-framework/datasource"
	"github.com/hashicorp/terraform-plugin-framework/datasource/schema"
	"github.com/hashicorp/terraform-plugin-log/tflog"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/client"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/constants"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/service/cloudbroker/vins/flattens"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/service/cloudbroker/vins/models"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/service/cloudbroker/vins/schemas"
)

// Ensure the implementation satisfies the expected interfaces.
var (
	_ datasource.DataSource = &dataSourceVINSExtNetList{}
)

func NewDataSourceVINSExtNetList() datasource.DataSource {
	return &dataSourceVINSExtNetList{}
}

// dataSourceVINSExtNetList is the data source implementation.
type dataSourceVINSExtNetList struct {
	client *client.Client
}

func (d *dataSourceVINSExtNetList) Read(ctx context.Context, req datasource.ReadRequest, resp *datasource.ReadResponse) {
	// Read Terraform configuration data into the model
	var state models.DataSourceVINSExtNetListModel
	resp.Diagnostics.Append(req.Config.Get(ctx, &state)...)
	if resp.Diagnostics.HasError() {
		tflog.Error(ctx, "Read dataSourceVINSExtNetList: Error get state")
		return
	}
	vinsID := uint64(state.VinsID.ValueInt64())
	tflog.Info(ctx, "Read dataSourceVINSExtNetList: got state successfully", map[string]any{"vins_id": vinsID})

	// Set timeouts
	readTimeout, diags := state.Timeouts.Read(ctx, constants.Timeout30s)
	resp.Diagnostics.Append(diags...)
	if resp.Diagnostics.HasError() {
		tflog.Error(ctx, "Read dataSourceVINSExtNetList: Error set timeout")
		return
	}
	tflog.Info(ctx, "Read dataSourceVINSExtNetList: set timeouts successfully", map[string]any{
		"vins_id":     vinsID,
		"readTimeout": readTimeout})

	ctx, cancel := context.WithTimeout(ctx, readTimeout)
	defer cancel()

	// Map response body to schema
	resp.Diagnostics.Append(flattens.VINSExtNetListDataSource(ctx, &state, d.client)...)
	if resp.Diagnostics.HasError() {
		tflog.Error(ctx, "Read dataSourceVINSExtNetList: Error flatten")
		return
	}

	// Set refreshed state
	resp.Diagnostics.Append(resp.State.Set(ctx, &state)...)
	if resp.Diagnostics.HasError() {
		tflog.Error(ctx, "Read dataSourceVINSExtNetList: Error set state")
		return
	}
	tflog.Info(ctx, "End read dataSourceVINSExtNetList", map[string]any{"vins_id": vinsID})
}

func (d *dataSourceVINSExtNetList) Schema(ctx context.Context, _ datasource.SchemaRequest, resp *datasource.SchemaResponse) {
	resp.Schema = schema.Schema{
		Attributes: schemas.MakeSchemaDataSourceVINSExtNetList(),
		Blocks: map[string]schema.Block{
			"timeouts": timeouts.Block(ctx),
		},
	}
}

func (d *dataSourceVINSExtNetList) Metadata(_ context.Context, req datasource.MetadataRequest, resp *datasource.MetadataResponse) {
	resp.TypeName = req.ProviderTypeName + "_cb_vins_ext_net_list"
}

// Configure adds the provider configured client to the data source.
func (d *dataSourceVINSExtNetList) Configure(ctx context.Context, req datasource.ConfigureRequest, resp *datasource.ConfigureResponse) {
	tflog.Info(ctx, "Get Configure dataSourceVINSExtNetList")
	d.client = client.DataSource(ctx, &req, resp)
	tflog.Info(ctx, "Getting Configure dataSourceVINSExtNetList successfully")
}
